# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.date_time_filter import DateTimeFilter
from ..types.http_validation_error import HttpValidationError
from ..types.int_range_filter import IntRangeFilter
from ..types.list_response_refund_external import ListResponseRefundExternal
from ..types.refund_external import RefundExternal
from ..types.refund_reason_enum import RefundReasonEnum
from ..types.refund_status_enum import RefundStatusEnum

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RefundsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        charge_id: typing.Optional[str] = OMIT,
        payment_intent_id: typing.Optional[str] = OMIT,
        amount_atom: typing.Optional[int] = OMIT,
        reason: typing.Optional[RefundReasonEnum] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> RefundExternal:
        """
        Parameters
        ----------
        charge_id : typing.Optional[str]
            Unique Identifier of the charge.

        payment_intent_id : typing.Optional[str]
            Unique Identifier of the payment_intent. Refund will be given on the latest charge of the payment_intent.

        amount_atom : typing.Optional[int]
            amount_atom that you want to refund.

        reason : typing.Optional[RefundReasonEnum]
            Reason of the refund.

        description : typing.Optional[str]
            Description of the refund.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefundExternal
            Successful Response

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        client.refunds.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "refunds/",
            method="POST",
            json={
                "charge_id": charge_id,
                "payment_intent_id": payment_intent_id,
                "amount_atom": amount_atom,
                "reason": reason,
                "description": description,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(RefundExternal, construct_type(type_=RefundExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self,
        *,
        page_number: typing.Optional[int] = OMIT,
        page_size: typing.Optional[int] = OMIT,
        sort_key: typing.Optional[str] = OMIT,
        sort_descending: typing.Optional[bool] = OMIT,
        created_at: typing.Optional[DateTimeFilter] = OMIT,
        expand: typing.Optional[typing.Sequence[str]] = OMIT,
        charge_id: typing.Optional[str] = OMIT,
        payment_intent_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        reason: typing.Optional[RefundReasonEnum] = OMIT,
        status: typing.Optional[RefundStatusEnum] = OMIT,
        amount_atom: typing.Optional[IntRangeFilter] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> SyncPager[RefundExternal]:
        """
        Parameters
        ----------
        page_number : typing.Optional[int]
            Page number

        page_size : typing.Optional[int]
            Page size

        sort_key : typing.Optional[str]
            Key name based on which data is sorted.

        sort_descending : typing.Optional[bool]
            Sort direction.

        created_at : typing.Optional[DateTimeFilter]
            Filter for selecting objects based on the range of creation dates. Includes options for equality, and greater than/less than comparisons.

        expand : typing.Optional[typing.Sequence[str]]
            Specifies which fields in the response should be expanded.

        charge_id : typing.Optional[str]
            Unique Identifier of the charge.

        payment_intent_id : typing.Optional[str]
            Unique Identifier of the payment_intent.

        customer_id : typing.Optional[str]
            Unique Identifier of the customer.

        reason : typing.Optional[RefundReasonEnum]
            Reason of the refund.

        status : typing.Optional[RefundStatusEnum]
            Status of the refund.

        amount_atom : typing.Optional[IntRangeFilter]
            This contains fields used for range filters on amount_atom field.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RefundExternal]
            Successful Response

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        response = client.refunds.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page_number = page_number if page_number is not None else 1
        _response = self._client_wrapper.httpx_client.request(
            "refunds/list",
            method="POST",
            json={
                "page_number": page_number,
                "page_size": page_size,
                "sort_key": sort_key,
                "sort_descending": sort_descending,
                "created_at": created_at,
                "expand": expand,
                "charge_id": charge_id,
                "payment_intent_id": payment_intent_id,
                "customer_id": customer_id,
                "reason": reason,
                "status": status,
                "amount_atom": amount_atom,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(ListResponseRefundExternal, construct_type(type_=ListResponseRefundExternal, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list(
                    page_number=page_number + 1,
                    page_size=page_size,
                    sort_key=sort_key,
                    sort_descending=sort_descending,
                    created_at=created_at,
                    expand=expand,
                    charge_id=charge_id,
                    payment_intent_id=payment_intent_id,
                    customer_id=customer_id,
                    reason=reason,
                    status=status,
                    amount_atom=amount_atom,
                    request_options=request_options,
                )
                _items = _parsed_response.data
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRefundsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        charge_id: typing.Optional[str] = OMIT,
        payment_intent_id: typing.Optional[str] = OMIT,
        amount_atom: typing.Optional[int] = OMIT,
        reason: typing.Optional[RefundReasonEnum] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> RefundExternal:
        """
        Parameters
        ----------
        charge_id : typing.Optional[str]
            Unique Identifier of the charge.

        payment_intent_id : typing.Optional[str]
            Unique Identifier of the payment_intent. Refund will be given on the latest charge of the payment_intent.

        amount_atom : typing.Optional[int]
            amount_atom that you want to refund.

        reason : typing.Optional[RefundReasonEnum]
            Reason of the refund.

        description : typing.Optional[str]
            Description of the refund.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RefundExternal
            Successful Response

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.refunds.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "refunds/",
            method="POST",
            json={
                "charge_id": charge_id,
                "payment_intent_id": payment_intent_id,
                "amount_atom": amount_atom,
                "reason": reason,
                "description": description,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(RefundExternal, construct_type(type_=RefundExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self,
        *,
        page_number: typing.Optional[int] = OMIT,
        page_size: typing.Optional[int] = OMIT,
        sort_key: typing.Optional[str] = OMIT,
        sort_descending: typing.Optional[bool] = OMIT,
        created_at: typing.Optional[DateTimeFilter] = OMIT,
        expand: typing.Optional[typing.Sequence[str]] = OMIT,
        charge_id: typing.Optional[str] = OMIT,
        payment_intent_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        reason: typing.Optional[RefundReasonEnum] = OMIT,
        status: typing.Optional[RefundStatusEnum] = OMIT,
        amount_atom: typing.Optional[IntRangeFilter] = OMIT,
        request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncPager[RefundExternal]:
        """
        Parameters
        ----------
        page_number : typing.Optional[int]
            Page number

        page_size : typing.Optional[int]
            Page size

        sort_key : typing.Optional[str]
            Key name based on which data is sorted.

        sort_descending : typing.Optional[bool]
            Sort direction.

        created_at : typing.Optional[DateTimeFilter]
            Filter for selecting objects based on the range of creation dates. Includes options for equality, and greater than/less than comparisons.

        expand : typing.Optional[typing.Sequence[str]]
            Specifies which fields in the response should be expanded.

        charge_id : typing.Optional[str]
            Unique Identifier of the charge.

        payment_intent_id : typing.Optional[str]
            Unique Identifier of the payment_intent.

        customer_id : typing.Optional[str]
            Unique Identifier of the customer.

        reason : typing.Optional[RefundReasonEnum]
            Reason of the refund.

        status : typing.Optional[RefundStatusEnum]
            Status of the refund.

        amount_atom : typing.Optional[IntRangeFilter]
            This contains fields used for range filters on amount_atom field.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RefundExternal]
            Successful Response

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.refunds.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        page_number = page_number if page_number is not None else 1
        _response = await self._client_wrapper.httpx_client.request(
            "refunds/list",
            method="POST",
            json={
                "page_number": page_number,
                "page_size": page_size,
                "sort_key": sort_key,
                "sort_descending": sort_descending,
                "created_at": created_at,
                "expand": expand,
                "charge_id": charge_id,
                "payment_intent_id": payment_intent_id,
                "customer_id": customer_id,
                "reason": reason,
                "status": status,
                "amount_atom": amount_atom,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(ListResponseRefundExternal, construct_type(type_=ListResponseRefundExternal, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list(
                    page_number=page_number + 1,
                    page_size=page_size,
                    sort_key=sort_key,
                    sort_descending=sort_descending,
                    created_at=created_at,
                    expand=expand,
                    charge_id=charge_id,
                    payment_intent_id=payment_intent_id,
                    customer_id=customer_id,
                    reason=reason,
                    status=status,
                    amount_atom=amount_atom,
                    request_options=request_options,
                )
                _items = _parsed_response.data
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
