# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.date_time_filter import DateTimeFilter
from ..types.http_validation_error import HttpValidationError
from ..types.list_response_transition_eligibility_external import ListResponseTransitionEligibilityExternal
from ..types.transition_eligibility_external import TransitionEligibilityExternal

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TransitionEligibilityClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        current_price_id: typing.Optional[str] = OMIT,
        target_price_id: typing.Optional[str] = OMIT,
        current_product_id: typing.Optional[str] = OMIT,
        target_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TransitionEligibilityExternal:
        """
        Create a new TransitionEligibility.

        Parameters
        ----------
        current_price_id : typing.Optional[str]
            Current price ID of the transition eligibility.

        target_price_id : typing.Optional[str]
            Target price ID of the transition eligibility.

        current_product_id : typing.Optional[str]
            Current product ID of the transition eligibility.

        target_product_id : typing.Optional[str]
            Target product ID of the transition eligibility.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransitionEligibilityExternal
            Successful Response

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        client.transition_eligibility.create()
        """
        _response = self._client_wrapper.httpx_client.request(
            "transition-eligibility/",
            method="POST",
            json={
                "current_price_id": current_price_id,
                "target_price_id": target_price_id,
                "current_product_id": current_product_id,
                "target_product_id": target_product_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(TransitionEligibilityExternal, construct_type(type_=TransitionEligibilityExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> TransitionEligibilityExternal:
        """
        Get a specific TransitionEligibility by id.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransitionEligibilityExternal
            Successful Response

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        client.transition_eligibility.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"transition-eligibility/{jsonable_encoder(id)}", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(TransitionEligibilityExternal, construct_type(type_=TransitionEligibilityExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: str,
        *,
        current_price_id: typing.Optional[str] = OMIT,
        target_price_id: typing.Optional[str] = OMIT,
        current_product_id: typing.Optional[str] = OMIT,
        target_product_id: typing.Optional[str] = OMIT,
        delete: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TransitionEligibilityExternal:
        """
        Update a specific TransitionEligibility.

        Parameters
        ----------
        id : str

        current_price_id : typing.Optional[str]
            Update the current price ID of the transition eligibility.

        target_price_id : typing.Optional[str]
            Update the target price ID of the transition eligibility.

        current_product_id : typing.Optional[str]
            Update the current product ID of the transition eligibility.

        target_product_id : typing.Optional[str]
            Update the target product ID of the transition eligibility.

        delete : typing.Optional[bool]
            Whether or not to delete this transition eligibility

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransitionEligibilityExternal
            Successful Response

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        client.transition_eligibility.update(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"transition-eligibility/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "current_price_id": current_price_id,
                "target_price_id": target_price_id,
                "current_product_id": current_product_id,
                "target_product_id": target_product_id,
                "delete": delete,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(TransitionEligibilityExternal, construct_type(type_=TransitionEligibilityExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific TransitionEligibility.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        client.transition_eligibility.delete(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"transition-eligibility/{jsonable_encoder(id)}", method="DELETE", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self,
        *,
        page_number: typing.Optional[int] = OMIT,
        page_size: typing.Optional[int] = OMIT,
        sort_key: typing.Optional[str] = OMIT,
        sort_descending: typing.Optional[bool] = OMIT,
        created_at: typing.Optional[DateTimeFilter] = OMIT,
        expand: typing.Optional[typing.Sequence[str]] = OMIT,
        is_deleted: typing.Optional[bool] = OMIT,
        current_price_id: typing.Optional[str] = OMIT,
        target_price_id: typing.Optional[str] = OMIT,
        current_product_id: typing.Optional[str] = OMIT,
        target_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[TransitionEligibilityExternal]:
        """
        List all TransitionEligibilities.

        Parameters
        ----------
        page_number : typing.Optional[int]
            Page number

        page_size : typing.Optional[int]
            Page size

        sort_key : typing.Optional[str]
            Key name based on which data is sorted.

        sort_descending : typing.Optional[bool]
            Sort direction.

        created_at : typing.Optional[DateTimeFilter]
            Filter for selecting objects based on the range of creation dates. Includes options for equality, and greater than/less than comparisons.

        expand : typing.Optional[typing.Sequence[str]]
            Specifies which fields in the response should be expanded.

        is_deleted : typing.Optional[bool]
            Filter transition eligibilities by is_deleted

        current_price_id : typing.Optional[str]
            Filter transition eligibilities by current price ID.

        target_price_id : typing.Optional[str]
            Filter transition eligibilities by target price ID.

        current_product_id : typing.Optional[str]
            Filter transition eligibilities by current product ID.

        target_product_id : typing.Optional[str]
            Filter transition eligibilities by target product ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[TransitionEligibilityExternal]
            Successful Response

        Examples
        --------
        from openpay.client import OpenPay

        client = OpenPay(
            token="YOUR_TOKEN",
        )
        response = client.transition_eligibility.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page_number = page_number if page_number is not None else 1
        _response = self._client_wrapper.httpx_client.request(
            "transition-eligibility/list",
            method="POST",
            json={
                "page_number": page_number,
                "page_size": page_size,
                "sort_key": sort_key,
                "sort_descending": sort_descending,
                "created_at": created_at,
                "expand": expand,
                "is_deleted": is_deleted,
                "current_price_id": current_price_id,
                "target_price_id": target_price_id,
                "current_product_id": current_product_id,
                "target_product_id": target_product_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(ListResponseTransitionEligibilityExternal, construct_type(type_=ListResponseTransitionEligibilityExternal, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list(
                    page_number=page_number + 1,
                    page_size=page_size,
                    sort_key=sort_key,
                    sort_descending=sort_descending,
                    created_at=created_at,
                    expand=expand,
                    is_deleted=is_deleted,
                    current_price_id=current_price_id,
                    target_price_id=target_price_id,
                    current_product_id=current_product_id,
                    target_product_id=target_product_id,
                    request_options=request_options,
                )
                _items = _parsed_response.data
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTransitionEligibilityClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        current_price_id: typing.Optional[str] = OMIT,
        target_price_id: typing.Optional[str] = OMIT,
        current_product_id: typing.Optional[str] = OMIT,
        target_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TransitionEligibilityExternal:
        """
        Create a new TransitionEligibility.

        Parameters
        ----------
        current_price_id : typing.Optional[str]
            Current price ID of the transition eligibility.

        target_price_id : typing.Optional[str]
            Target price ID of the transition eligibility.

        current_product_id : typing.Optional[str]
            Current product ID of the transition eligibility.

        target_product_id : typing.Optional[str]
            Target product ID of the transition eligibility.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransitionEligibilityExternal
            Successful Response

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transition_eligibility.create()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "transition-eligibility/",
            method="POST",
            json={
                "current_price_id": current_price_id,
                "target_price_id": target_price_id,
                "current_product_id": current_product_id,
                "target_product_id": target_product_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(TransitionEligibilityExternal, construct_type(type_=TransitionEligibilityExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TransitionEligibilityExternal:
        """
        Get a specific TransitionEligibility by id.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransitionEligibilityExternal
            Successful Response

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transition_eligibility.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"transition-eligibility/{jsonable_encoder(id)}", method="GET", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(TransitionEligibilityExternal, construct_type(type_=TransitionEligibilityExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: str,
        *,
        current_price_id: typing.Optional[str] = OMIT,
        target_price_id: typing.Optional[str] = OMIT,
        current_product_id: typing.Optional[str] = OMIT,
        target_product_id: typing.Optional[str] = OMIT,
        delete: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TransitionEligibilityExternal:
        """
        Update a specific TransitionEligibility.

        Parameters
        ----------
        id : str

        current_price_id : typing.Optional[str]
            Update the current price ID of the transition eligibility.

        target_price_id : typing.Optional[str]
            Update the target price ID of the transition eligibility.

        current_product_id : typing.Optional[str]
            Update the current product ID of the transition eligibility.

        target_product_id : typing.Optional[str]
            Update the target product ID of the transition eligibility.

        delete : typing.Optional[bool]
            Whether or not to delete this transition eligibility

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TransitionEligibilityExternal
            Successful Response

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transition_eligibility.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"transition-eligibility/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "current_price_id": current_price_id,
                "target_price_id": target_price_id,
                "current_product_id": current_product_id,
                "target_product_id": target_product_id,
                "delete": delete,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(TransitionEligibilityExternal, construct_type(type_=TransitionEligibilityExternal, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a specific TransitionEligibility.

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.transition_eligibility.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"transition-eligibility/{jsonable_encoder(id)}", method="DELETE", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self,
        *,
        page_number: typing.Optional[int] = OMIT,
        page_size: typing.Optional[int] = OMIT,
        sort_key: typing.Optional[str] = OMIT,
        sort_descending: typing.Optional[bool] = OMIT,
        created_at: typing.Optional[DateTimeFilter] = OMIT,
        expand: typing.Optional[typing.Sequence[str]] = OMIT,
        is_deleted: typing.Optional[bool] = OMIT,
        current_price_id: typing.Optional[str] = OMIT,
        target_price_id: typing.Optional[str] = OMIT,
        current_product_id: typing.Optional[str] = OMIT,
        target_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[TransitionEligibilityExternal]:
        """
        List all TransitionEligibilities.

        Parameters
        ----------
        page_number : typing.Optional[int]
            Page number

        page_size : typing.Optional[int]
            Page size

        sort_key : typing.Optional[str]
            Key name based on which data is sorted.

        sort_descending : typing.Optional[bool]
            Sort direction.

        created_at : typing.Optional[DateTimeFilter]
            Filter for selecting objects based on the range of creation dates. Includes options for equality, and greater than/less than comparisons.

        expand : typing.Optional[typing.Sequence[str]]
            Specifies which fields in the response should be expanded.

        is_deleted : typing.Optional[bool]
            Filter transition eligibilities by is_deleted

        current_price_id : typing.Optional[str]
            Filter transition eligibilities by current price ID.

        target_price_id : typing.Optional[str]
            Filter transition eligibilities by target price ID.

        current_product_id : typing.Optional[str]
            Filter transition eligibilities by current product ID.

        target_product_id : typing.Optional[str]
            Filter transition eligibilities by target product ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[TransitionEligibilityExternal]
            Successful Response

        Examples
        --------
        import asyncio

        from openpay.client import AsyncOpenPay

        client = AsyncOpenPay(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.transition_eligibility.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        page_number = page_number if page_number is not None else 1
        _response = await self._client_wrapper.httpx_client.request(
            "transition-eligibility/list",
            method="POST",
            json={
                "page_number": page_number,
                "page_size": page_size,
                "sort_key": sort_key,
                "sort_descending": sort_descending,
                "created_at": created_at,
                "expand": expand,
                "is_deleted": is_deleted,
                "current_price_id": current_price_id,
                "target_price_id": target_price_id,
                "current_product_id": current_product_id,
                "target_product_id": target_product_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(ListResponseTransitionEligibilityExternal, construct_type(type_=ListResponseTransitionEligibilityExternal, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list(
                    page_number=page_number + 1,
                    page_size=page_size,
                    sort_key=sort_key,
                    sort_descending=sort_descending,
                    created_at=created_at,
                    expand=expand,
                    is_deleted=is_deleted,
                    current_price_id=current_price_id,
                    target_price_id=target_price_id,
                    current_product_id=current_product_id,
                    target_product_id=target_product_id,
                    request_options=request_options,
                )
                _items = _parsed_response.data
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
